# Native python packages
import os

import click

# Chronix2grid modules
from chronix2grid.generation import generate_chronics as gen
from chronix2grid.kpi import main as kpis

# ==============================================================
## CONSTANT VARIABLES
@click.command()
@click.option('--case', default='case118_l2rpn', help='case folder to base generation on')
@click.option('--start-date', default='2012-01-01', help='Start date to generate chronics')
@click.option('--weeks', default=4, help='Number of weeks to generate')
@click.option('--n_scenarios', default=1, help='Number of scenarios to generate')
@click.option('--mode', default='LRTK', help='Steps to execute : L for loads only (and KPI); R(K) for renewables (and KPI) only; LRTK for all generation')
@click.option('--root-folder', default=os.path.normpath(os.getcwd()), help='root of all file generation and input')
def generate(case, start_date, weeks, n_scenarios, mode, root_folder):
    INPUT_FOLDER = os.path.join(root_folder, 'generation', 'input')
    OUTPUT_FOLDER = os.path.join(root_folder, 'generation', 'output')

    KPI_INPUT_FOLDER = os.path.join(root_folder, "kpi", "input")
    IMAGES_FOLDER = os.path.join(root_folder, "kpi", "images")
    # Folders are specific to studied case
    output_folder = os.path.join(OUTPUT_FOLDER, case)
    images_folder = os.path.join(IMAGES_FOLDER, case)

    # Reading parameters
    year, params, loads_charac, prods_charac, load_weekly_pattern, solar_pattern, lines, params_opf = gen.read_configuration(
        INPUT_FOLDER, case, start_date, weeks)

    # Chronic generation
    if 'L' in mode or 'R' in mode:
        gen.main(case, year, n_scenarios, params, INPUT_FOLDER, output_folder, prods_charac, loads_charac, lines, solar_pattern, load_weekly_pattern, params_opf, mode)

    # KPI formatting and computing
    if 'R' in mode and 'K' in mode and 'T' not in mode:
        # Get and format solar and wind on all timescale, then compute KPI and save plots
        wind_solar_only = True
        os.makedirs(IMAGES_FOLDER, exist_ok=True)
        kpis.main(KPI_INPUT_FOLDER, INPUT_FOLDER, output_folder, images_folder, year, case, n_scenarios, wind_solar_only, params, loads_charac, prods_charac)

    elif 'T' in mode and 'K' in mode:
        # Get and format dispatched chronics, then compute KPI and save plots
        wind_solar_only = False
        os.makedirs(IMAGES_FOLDER, exist_ok=True)
        kpis.main(KPI_INPUT_FOLDER, INPUT_FOLDER, output_folder, images_folder, year, case, n_scenarios, wind_solar_only, params, loads_charac, prods_charac)


if __name__ == "__main__":
    generate()



# Python built-in modules
import warnings
import os
import json

# Chronix2grid modules
from .preprocessing.pivot_KPI import pivot_format
from .deterministic.kpis import EconomicDispatchValidator


def main(kpi_input_folder, generation_input_folder, generation_output_folder, images_repo, year, case, n_scenarios, wind_solar_only, params,
         loads_charac, prods_charac):
    """
        This is the main function for KPI computation. It formats synthetic and reference chronics and then computes KPIs on it with 2 modes (wind solar and load only or full energy mix). It saves plots in png and html files. It writes a json output

        Parameters
        ----------
        kpi_input_folder (str): path to folder of kpi inputs, which contains paramsKPI.json and benchmark folders. paramsKPI.json tells which benchmark to read as reference
        generation_input_folder (str): path to the input folder of generation module (to read chronics before dispatch)
        generation_output_folder (str): path to the output folder of generation module (to read chronics after dispatch)
        images_repo (str): path to the images folder in which KPI plots will be saved
        year (int): year in which chronics have been generated
        case (str): identify the studied case for chronics generation, such as l2rpn_118
        n_scenario (int): number of scenarios to consider (KPIs are computed for each scenario succesively)
        wind_solar_only (boolean): True if the generated chronics contain only wind, solar and load chronics, False otherwise
        params (dict): configuration params computed from params.json, such as timestep or mesh characteristics
        loads_charac (pandas.DataFrame): characteristics of loads node such as Pmax, type of demand and region
        prods_charac (pandas.DataFrame): characteristics of generators such as Pmax, carrier and region

    """


    print('=====================================================================================================================================')
    print('================================================= KPI GENERATION  ===================================================================')
    print('=====================================================================================================================================')

    warnings.filterwarnings("ignore")


    # Create single zone if no zone is given
    if 'zone' not in prods_charac.columns:
        prods_charac['zone'] = 'R1'
        loads_charac['zone'] = 'R1'

    ## Format and compute KPI for each scenario
    for scenario_num in range(n_scenarios):
        print('Scenario '+str(scenario_num)+'...')

        # Return Warning if KPIs are not computed on full year. Yet, the computation will work
        if params['weeks'] != 52:
            print('Warning: KPI are incomplete. Computation has been made on '+str(params['weeks'])+' weeks, but are meant to be computed on 52 weeks')

        # Read reference and synthetic chronics, but also KPI configuration, in pivot format. 2 modes: with or without full dispatch
        if wind_solar_only:
            # Get reference and synthetic dispatch and loads
            ref_dispatch, ref_consumption, syn_dispatch, syn_consumption, monthly_pattern, hours = pivot_format(generation_input_folder,
                                                                                                            kpi_input_folder,
                                                                                                            year,
                                                                                                            scenario_num,
                                                                                                            prods_charac,
                                                                                                            loads_charac,
                                                                                                            wind_solar_only,
                                                                                                            params, case)
            ref_prices = None
            prices = None
        else:
            # Get reference and synthetic dispatch and loads
            ref_dispatch, ref_consumption, syn_dispatch, syn_consumption, monthly_pattern, hours, ref_prices, prices = pivot_format(
                                                                                                generation_output_folder,
                                                                                                kpi_input_folder,
                                                                                                year,
                                                                                                scenario_num,
                                                                                                prods_charac,
                                                                                                loads_charac,
                                                                                                wind_solar_only,
                                                                                                params, case)


        ## Start and Run Economic dispatch validator
        # -- + -- + -- + -- + -- + -- + --
        print ('(1) Computing KPI\'s...')
        dispatch_validator = EconomicDispatchValidator(ref_consumption,
                                                       syn_consumption,
                                                       ref_dispatch,
                                                       syn_dispatch,
                                                       year,
                                                       scenario_num,
                                                       images_repo,
                                                       prods_charac=prods_charac,
                                                       loads_charac=loads_charac,
                                                       ref_prices=ref_prices,
                                                       syn_prices=prices)


        # Compute dispatch temporal view
        if wind_solar_only:
            max_col = 1
        else:
            max_col = 2
        dispatch_validator.plot_carriers_pw(curve='reference', stacked=True, max_col_splot=max_col, save_html=True,
                                            wind_solar_only=wind_solar_only)
        dispatch_validator.plot_carriers_pw(curve='synthetic', stacked=True, max_col_splot=max_col, save_html=True,
                                            wind_solar_only=wind_solar_only)

        # Get Load KPI
        dispatch_validator.load_kpi()

        # Get Wind KPI
        dispatch_validator.wind_kpi()

        # Get Solar KPI
        dispatch_validator.solar_kpi(monthly_pattern=monthly_pattern, hours=hours)

        # Wind - Solar KPI
        dispatch_validator.wind_load_kpi()

        # These KPI only if dispatch has been made
        if not wind_solar_only:
            # Get Energy Mix
            dispatch_validator.energy_mix()

            # Get Hydro KPI
            dispatch_validator.hydro_kpi()

            # Get Nuclear KPI
            dispatch_validator.nuclear_kpi()

            # Get Thermal KPI
            dispatch_validator.thermal_kpi()
            dispatch_validator.thermal_load_kpi()


        # Write json output file
        # -- + -- + -- + -- + --
        print ('(2) Generating json output file...')

        kpi_output_folder = os.path.join(kpi_input_folder, os.pardir, 'output', str(year))
        if not os.path.exists(kpi_output_folder):
            os.makedirs(kpi_output_folder)
        kpi_output_folder = os.path.join(kpi_output_folder,'Scenario_' + str(scenario_num))
        if not os.path.exists(kpi_output_folder):
            os.makedirs(kpi_output_folder)

        with open(os.path.join(kpi_output_folder,'ec_validator_output.json'), 'w') as json_f:
            json.dump(dispatch_validator.output, json_f)

        print ('-Done-\n')


import re
import os
import numpy as np
import pandas as pd
from scipy.interpolate import interp1d


def make_generation_input_output_directories(input_folder, case, year, output_folder):

    dispatch_input_folder_case = os.path.join(input_folder, case, 'dispatch')
    dispatch_input_folder = os.path.join(dispatch_input_folder_case, str(year))
    dispatch_output_folder = os.path.join(output_folder, str(year))

    os.makedirs(dispatch_input_folder_case, exist_ok=True)
    os.makedirs(dispatch_input_folder, exist_ok=True)
    os.makedirs(dispatch_output_folder, exist_ok=True)

    return dispatch_input_folder, dispatch_input_folder_case, dispatch_output_folder


def generate_coarse_noise(params, data_type):
    """
    This function generates a spatially and temporally correlated noise.
    Because it may take a lot of time to compute a correlated noise on
    a too fine mesh, we recommend to first compute a correlated signal
    on a coarse mesh, and then use the interpolation function.

    Input:
        params: (dict) Defines the mesh dimensions and
            precision. Also define the correlation scales

    Output:
        (np.array) 3D autocorrelated noise
    """

    # Get computation domain size
    Lx = params['Lx']
    Ly = params['Ly']
    T = params['T']

    # Get the decay parameter for each dimension
    dx_corr = params['dx_corr']
    dy_corr = params['dy_corr']
    dt_corr = params[data_type + '_corr']

    # Compute number of element in each dimension
    Nx_comp = int(Lx // dx_corr + 1)
    Ny_comp = int(Ly // dy_corr + 1)
    Nt_comp = int(T // dt_corr + 1)

    # Generate gaussian noise input·
    output = np.random.normal(0, 1, (Nx_comp, Ny_comp, Nt_comp))

    return output

def interpolate_noise(computation_noise, params, locations, time_scale):
    """
    This interpolates an autocarrelated noise mesh, to make it more granular.

    Input:
        computation_noise: (np.array) Autocorrelated signal computed on a coarse mesh
        params: (dict) Defines the mesh dimensions and
            precision. Also define the correlation scales
        locations: (dict) Defines the location of the points of interest in the domain

    Output:
        (dict of np.array) returns one time series per location mentioned in dict locations
    """

    # Get computation domain size
    Lx = params['Lx']
    Ly = params['Ly']
    T = params['T']

    # Get the decay parameter for each dimension
    dx_corr = params['dx_corr']
    dy_corr = params['dy_corr']
    dt_corr = time_scale

    # Compute number of element in each dimension
    Nx_comp = int(Lx // dx_corr + 1)
    Ny_comp = int(Ly // dy_corr + 1)
    Nt_comp = int(T // dt_corr + 1)

    # Get interpolation temporal mesh size
    dt = params['dt']

    # Compute number of element in each dimension
    Nt_inter = T // dt + 1

    # Get coordinates
    x, y = locations

    # Get coordinates of closest points in the coarse mesh
    x_minus = int(x // dx_corr)
    x_plus = int(x // dx_corr + 1)
    y_minus = int(y // dy_corr)
    y_plus = int(y // dy_corr + 1)

    # 1st step : spatial interpolation

    # Initialize output
    output = np.zeros(Nt_comp)

    # Initialize sum of distances
    dist_tot = 0

    # For every close point, add the corresponding time series, weighted by the inverse
    # of the distance between them
    for x_neighbor in [x_minus, x_plus]:
        for y_neighbor in [y_minus, y_plus]:
            dist = 1 / (np.sqrt((x - dx_corr * x_neighbor) ** 2 + (y - dy_corr * y_neighbor) ** 2) + 1)
            output += dist * computation_noise[x_neighbor, y_neighbor, :]
            dist_tot += dist
    output /= dist_tot

    # 2nd step : temporal quadratic interpolation
    t_comp = np.linspace(0, int(T), int(Nt_comp), endpoint=True)
    t_inter = np.linspace(0, int(T), int(Nt_inter), endpoint=True)
    if Nt_comp == 2:
        f2 = interp1d(t_comp, output, kind='linear')
    elif Nt_comp == 3:
        f2 = interp1d(t_comp, output, kind='quadratic')
    elif Nt_comp > 3:
        f2 = interp1d(t_comp, output, kind='cubic')

    if Nt_comp >= 2:
        output = f2(t_inter)

    return output

def natural_keys(text):
    return int([ c for c in re.split('(\d+)', text) ][1])


# Native python libraries
import os
import json

# Other Python libraries
import pandas as pd
import numpy as np
from datetime import timedelta


# Libraries developed for this module
from .consumption import generate_load as gen_loads
from .renewable import generate_solar_wind as gen_enr
from .thermal import generate_dispatch as gen_dispatch
from .dispatch import utils as du
from .dispatch import EconomicDispatch as ec
from . import generation_utils as gu

def read_configuration(input_folder, case, start_date, weeks):
    """
    This functions reads the detailed parameters of the generation in params.json, but also the case settup through files prods_charac.csv, loads_charac.csv and lines.csv
    It processes it and returns usable format for each of those parameters

    Parameters
    ----------
    input_folder (string): peth of folder where inputs are stored
    case (str): name of case to study (must be a folder within input_folder)
    start_date (str): string containing start date of geneiration (recommended format is YYYY-MM-DD)
    weeks (int): number of weeks on which to generate the chronics

    Returns
    -------
    int: year of generated chronics (if weeks>52, takes the most ancient year)
    dict: dictionnary with parameters, including a formatting of params.json
    pandas.DataFrame: characteristics of generators such as Pmax, carrier and region
    pandas.DataFrame: characteristics of loads node such as Pmax, type of demand and region
    pandas.DataFrame: normalized weekly pattern of load, used as reference for load generation
    pandas.DataFrame: normalized yearly solar production pattern, used as reference for solar chronics generation
    pandas.DataFrame: characteristics of lines

    """

    # Read Json parameters
    print('Importing parameters ...')
    json1_file = open(os.path.join(input_folder, case, 'params.json'))
    json1_str = json1_file.read()
    params = json.loads(json1_str)
    for key, value in params.items():
        try:
            params[key] = float(value)
        except ValueError:
            params[key] = pd.to_datetime(value, format='%Y-%m-%d')

    with open(os.path.join(input_folder, case, 'params_opf.json'), 'r') as opf_param_json:
        params_opf = json.load(opf_param_json)

    # Compute date and time parameters
    start_date = pd.to_datetime(start_date, format='%Y-%m-%d')
    params['weeks'] = weeks
    params['start_date'] = start_date
    year = start_date.year
    params['end_date'] = params['start_date'] + timedelta(days=7 * int(weeks)) - timedelta(minutes=params['dt'])
    params['T'] = int(pd.Timedelta(params['end_date'] - params['start_date']).total_seconds() // (60))
    # Nt_inter = int(params['T'] // params['dt'] + 1)

    # Import loads_charac.csv, prods_charac.csv and lines.csv in desired case
    print('Importing loads prods and lines parameters ...')
    try:
        loads_charac = pd.read_csv(os.path.join(input_folder, case, 'loads_charac.csv'), sep = ',')
        names = loads_charac['name']   # to generate error if separator is wrong
        prods_charac = pd.read_csv(os.path.join(input_folder, case, 'prods_charac.csv'), sep = ',')
        lines = pd.read_csv(os.path.join(input_folder, case, 'lines_names.csv'), sep=',')
    except:
        loads_charac = pd.read_csv(os.path.join(input_folder, case, 'loads_charac.csv'), sep=';')
        prods_charac = pd.read_csv(os.path.join(input_folder, case, 'prods_charac.csv'), sep=';')
        lines = pd.read_csv(os.path.join(input_folder, case, 'lines_names.csv'), sep=';')

    # Importing weekly patterns
    load_weekly_pattern = pd.read_csv(os.path.join(input_folder, 'patterns', 'load_weekly_pattern.csv'))
    solar_pattern = np.load(os.path.join(input_folder, 'patterns', 'solar_pattern.npy'))

    return year, params, loads_charac, prods_charac, load_weekly_pattern, solar_pattern, lines, params_opf


# Call generation scripts n_scenario times with dedicated random seeds
def main(case, year, n_scenarios, params, input_folder, output_folder,
         prods_charac, loads_charac, lines, solar_pattern, load_weekly_pattern,
         params_opf, mode='LRTK'):
    """
    Main function for chronics generation. It works with three steps: load generation, renewable generation (solar and wind) and then dispatch computation to get the whole energy mix

    Parameters
    ----------
    case (str): name of case to study (must be a folder within input_folder)
    year (int): year of generation
    n_scenarios (int): number of desired scenarios to generate for the same timescale
    params (dict): parameters of generation, as returned by function chronix2grid.generation.generate_chronics.read_configuration
    input_folder (str): path of folder containing inputs
    output_folder (str): path where outputs will be written (intermediate folder case/year/scenario will be used)
    prods_charac (pandas.DataFrame): as returned by function chronix2grid.generation.generate_chronics.read_configuration
    loads_charac (pandas.DataFrame): as returned by function chronix2grid.generation.generate_chronics.read_configuration
    lines (pandas.DataFrame): as returned by function chronix2grid.generation.generate_chronics.read_configuration
    solar_pattern (pandas.DataFrame): as returned by function chronix2grid.generation.generate_chronics.read_configuration
    load_weekly_pattern (pandas.DataFrame): as returned by function chronix2grid.generation.generate_chronics.read_configuration
    mode (str): options to launch certain parts of the generation process : L load R renewable T thermal


    Returns
    -------

    """

    print('=====================================================================================================================================')
    print('============================================== CHRONICS GENERATION ==================================================================')
    print('=====================================================================================================================================')

    ## Random seeds:  Make sure the seeds are the same, whether computation is parallel or sequential
    seeds = [np.random.randint(low=0, high=2 ** 31) for _ in range(n_scenarios)]

    ## Folder settings

    dispatch_input_folder, dispatch_input_folder_case, dispatch_output_folder = gu.make_generation_input_output_directories(input_folder, case, year, output_folder)

    dispatcher = ec.init_dispatcher_from_config(params_opf["grid_path"], input_folder)

    ## Launch proper scenarios generation
    for i, seed in enumerate(seeds):
        scenario_name = f'Scenario_{i}'
        scenario_dispatch_input_folder = os.path.join(dispatch_input_folder, scenario_name)

        print("================ Generating "+scenario_name+" ================")
        if 'L' in mode:
            load, load_forecasted = gen_loads.main(scenario_dispatch_input_folder, seed, params, loads_charac, load_weekly_pattern, write_results = True)

        if 'R' in mode:
            prod_solar, prod_solar_forecasted, prod_wind, prod_wind_forecasted = gen_enr.main(scenario_dispatch_input_folder, seed, params, prods_charac, solar_pattern, write_results = True)
        if 'T' in mode:
            input_scenario_folder, output_scneario_folder = du.make_scenario_input_output_directories(
                dispatch_input_folder, dispatch_output_folder, scenario_name)

            prods = pd.concat([prod_solar, prod_wind], axis=1)
            res_names = dict(wind=prod_wind.columns, solar=prod_solar.columns)
            dispatcher.chronix_scenario = ec.ChroniXScenario(load, prods, res_names,
                                                             scenario_name)

            dispatch_results = gen_dispatch.main(dispatcher, input_scenario_folder,
                                                 output_scneario_folder,
                                                 seed, params_opf)
        print('\n')
    return


if __name__ == '__main__':
    CASE = 'case118_l2rpn'
    INPUT_FOLDER = os.path.abspath('input')
    OUTPUT_FOLDER = os.path.abspath('output')
    # Detailed configuration to set in <INPUT_FOLDER>/<CASE>/params.json
    start_date = "2012-01-01"
    weeks = 1
    n_scenarios = 1
    year, params, loads_charac, prods_charac, load_weekly_pattern, solar_pattern, lines, params_opf = read_configuration(INPUT_FOLDER, CASE, start_date, weeks)

    main(CASE, year, n_scenarios, params, INPUT_FOLDER, OUTPUT_FOLDER, prods_charac, loads_charac, lines, solar_pattern, load_weekly_pattern, params_opf)



import pandas as pd
import os
import numpy as np

def EnergyMix_AprioriChecker(env118_withoutchron,Target_EM_percentage, PeakLoad, AverageLoad, CapacityFactor ):
    # # Check the Energy Mix apriori

    # ### Target Energy mix 
    # when summing over scenarios
    # 

    # In[132]:


    #print('\n the target energy mix is')
    #print(Target_EM_percentage)


    # ### Check initial capacity mix

    # In[139]:


    Mix_df=pd.DataFrame({'name':env118_withoutchron.name_gen,'type':env118_withoutchron.gen_type,'pmax':env118_withoutchron.gen_pmax})
    TotalCapacity=Mix_df['pmax'].sum()
    CapacityPerType=Mix_df.groupby(['type'])['pmax'].agg('sum')

    #CapacityNonRenewable=CapacityPerType['thermal']+CapacityPerType['nuclear']+CapacityPerType['hydro']
    CapacityMix=np.round((CapacityPerType/TotalCapacity*100)*10)/10
    CapacityMix.name='capacity_mix'
    #print('\n Total capacity:'+str(np.round(TotalCapacity)))
    #print('Total non renewable capacity:'+str(np.round(CapacityNonRenewable)))
    #print(CapacityPerType)
    #print('\n the capacity mix is:')
    #print(CapacityMix)


    # ### Considering capacity factors

    # We need to define capacity factors then to estimate the share really produced.
    # We can consider that every renewable production than can be produced will be used.
    # According to https://bilan-electrique-2019.rte-france.com/, we can take a capacity factor of:
    #         - 15% for solar
    #         - 25% for wind
    #         
    # We can then consider that nuclear is almost always working near its full capacity, so a capacity factor of 0.95
    # 
    # Then for hydro, the pmax is changing over a year so it is not easy to estimate. 
    # But the storage we have in hydro will be consume anyway during the year anyway. As an indicator in France for dumps in 2019:
    #     - the max hyrdo was 6.5GW (so close to max capacity)
    #     - on average the production was 2.15 GW
    #     - so a capacity factor of 30%
    # 
    # For thermal, we can reach its max capacity when ever needed, if its fast enough. So there is no capacity factor so too speak, but just need to have the right residual capcity to go above peak net demand. 
    # 

    # In[140]:


    #print('\n the capacity factors are:')
    #print(CapacityFactor)


    # In[141]:


    Capacity=pd.concat([Target_EM_percentage, CapacityPerType, CapacityMix, CapacityFactor], axis=1)
    #Capacity['revised_pmax']=np.round((Capacity['pmax'].values/Capacity['capacity_factor'].values*100)*10)/10
    #print("\n revised capacities after taking into account capacity factor")
    #print(Capacity)


    # Finally thermal should be scaled so that nuclear + thermal deals with peak demand. 
    # Peak Demand below is 4200MW

    # In[142]:
    
    EnergyAPriori=Capacity['pmax']/100*Capacity['capacity_factor']
    EnergyAPriori

    MaxMixEnergyNoThermal=EnergyAPriori/EnergyAPriori.sum()*100
    MaxMixEnergyNoThermal

    thermalMix=Target_EM_percentage.loc['thermal'].values
    MaxMixEnergyWithThermal=MaxMixEnergyNoThermal*(1-thermalMix/100)
    MaxMixEnergyWithThermal['thermal']=thermalMix

    Capacity['Apriori_energy_mix']=Capacity['capacity_factor']*Capacity['pmax']/AverageLoad
    
    MixShareNoThermal=Capacity['Apriori_energy_mix'].sum()
    if (MixShareNoThermal>=100):
        Capacity['Apriori_energy_mix']['hydro']=Capacity['Apriori_energy_mix']['hydro']-(MixShareNoThermal-100)
        Capacity['Apriori_energy_mix']['thermal']=0
    else:
        Capacity['Apriori_energy_mix']['thermal']=100-MixShareNoThermal

    Capacity['revised_pmax']=Capacity['target_energy_mix']/Capacity['Apriori_energy_mix']*Capacity['pmax']
    Capacity['revised_pmax']['thermal']=PeakLoad-Capacity['revised_pmax']['nuclear']
    #print("\n revised thermal capacity")
    #print(Capacity)
    
    
    error=np.abs(Capacity['target_energy_mix']-Capacity['Apriori_energy_mix']).sum()
    print('Warning: the differences in your target energy mix and you energy mix a priori are: ' + str(error)+'%')

    return Capacity








def Ramps_Pmax_Pmin_APrioriCheckers(env118_withoutchron,Capacity, chronics_path_gen,losses_pct,expected_PeakLoad):



    isThermalInTrouble=False #check that we can satisfy peak demand with thermal and nuclear
    isNuclearInTrouble=False #check that  nuclear pMax is always above min net load, or otherwise that nuclear max ramp down is greater than net load max ramp down
    IsRampUpInTrouble=False #check that thermal and nuclear ramp up is greater than the ramp up of the net demand
    IsRampDownInTrouble=False #check that hydro and nuclear ramp down is greater than the ramp down of the net demand


    # Create dataFrames per type of chronics

    # In[19]:


    #Create DataFrames
    Load_df=pd.DataFrame()
    Load_net=pd.DataFrame()
    Wind_df=pd.DataFrame()
    Solar_df=pd.DataFrame()

    fileList=[f for f in os.listdir(chronics_path_gen) if not f.startswith('.')]
    for subpath in fileList:
        # Load consumption and prod
        this_path = os.path.join(chronics_path_gen, subpath)
        load_p = pd.read_csv(os.path.join(this_path, 'load_p.csv.bz2'), sep = ';')
        prod_p = pd.read_csv(os.path.join(this_path, 'prod_p.csv.bz2'), sep = ';')

       # Retrieve wind and solar from prod_p (Balthazar's generator)
        prod_p_wind = prod_p[[el for i, el in enumerate(env118_withoutchron.name_gen) if env118_withoutchron.gen_type[i] in ["wind"]]]
        total_p_wind=prod_p_wind.sum(axis=1)
        prod_p_solar = prod_p[[el for i, el in enumerate(env118_withoutchron.name_gen) if env118_withoutchron.gen_type[i] in ["solar"]]]
        total_p_solar=prod_p_solar.sum(axis=1) 

        total_renew = pd.concat([total_p_wind, total_p_solar], axis=1).sum(axis=1)    

        # Compensate the reactive part in loads
        #load_ = load_p.copy() * (1 + losses_pct/100)
        load_ = load_p.sum(axis=1)
        load_ = load_*(1 + losses_pct/100)

        # Demand for OPF (total - renewable)
        agg_load_without_renew = (load_ - total_renew).to_frame()
        Load_df[subpath]=load_
        Wind_df[subpath]=total_p_wind
        Solar_df[subpath]=total_p_solar

    Load_net=Load_df-Wind_df-Solar_df


    # ## Check renewable energy mix share

    # In[153]:

    MaxSolar=Solar_df.max().max()
    MaxWind=Wind_df.max().max()
    
    #Energy Mix
    TotalLoad=Load_df.sum().sum()
    TotalWind=Wind_df.sum().sum()
    TotalSolar=Solar_df.sum().sum()

    WindShare=np.round((TotalWind/TotalLoad*100)*10)/10
    SolarShare=np.round((TotalSolar/TotalLoad*100)*10)/10

    print('\n the wind share is '+str(WindShare))
    print('the wind share was expected to be '+str(Capacity['target_energy_mix']['wind']))
    print('the solar share is '+str(SolarShare))
    print('the solar share was expected to be '+str(Capacity['target_energy_mix']['solar']))


    # In[154]:


    df_stats=pd.DataFrame(Load_net.values.flatten()).describe()
    df_stats.columns=['Load_net']

    #df_stats['Load_net']=pd.DataFrame(Load_net.values.flatten()).describe()
    df_stats['Solar']=pd.DataFrame(Solar_df.values.flatten()).describe()
    df_stats['Wind']=pd.DataFrame(Wind_df.values.flatten()).describe()
    df_stats['Renewable']=pd.DataFrame(Wind_df.values.flatten()+Solar_df.values.flatten()).describe()
    df_stats['Load']=pd.DataFrame(Load_df.values.flatten()).describe()

    df_stats


    # In[159]:


    print('\n the max load is '+str(df_stats['Load']['max']))
    print('the expected peak load was '+str(expected_PeakLoad))
    print('\n the max net load is '+str(df_stats['Load_net']['max']))

    toleranceThreshold=0.02
    isThermalInTrouble=(abs(expected_PeakLoad-df_stats['Load']['max'])/expected_PeakLoad > toleranceThreshold)

    print('WARNING: if they differ by more than 2%, we should change the capacity of thermal productions')


    # In[170]:


    Ramps_gen_df=pd.DataFrame({'name':env118_withoutchron.name_gen,'type':env118_withoutchron.gen_type,
                               'rampUp':env118_withoutchron.gen_max_ramp_up,'rampDown':env118_withoutchron.gen_max_ramp_down})
    Ramps_genType_df=Ramps_gen_df.groupby(['type'])['rampUp','rampDown'].agg('sum')
    Ramps_genType_df


    # In[157]:


    df_stats_ramps=pd.DataFrame(Load_net.diff().values.flatten()).describe()
    df_stats_ramps.columns=['Load_net']

    #df_stats['Load_net']=pd.DataFrame(Load_net.values.flatten()).describe()
    df_stats_ramps['Solar']=pd.DataFrame(Solar_df.diff().values.flatten()).describe()
    df_stats_ramps['Wind']=pd.DataFrame(Wind_df.diff().values.flatten()).describe()
    df_stats_ramps['Renewable']=pd.DataFrame(Wind_df.diff().values.flatten()+Solar_df.values.flatten()).describe()
    df_stats_ramps['Load']=pd.DataFrame(Load_df.diff().values.flatten()).describe()

    df_stats_ramps


    # In[178]:


    isNuclearPmaxInTrouble=(df_stats['Load_net']['min']<Capacity['pmax']['nuclear'])
    isNuclearRampInTrouble=(df_stats_ramps['Load_net']['max']>Ramps_genType_df['rampDown']['nuclear'])
    isNuclearInTrouble=(isNuclearPmaxInTrouble and isNuclearRampInTrouble)

    print('\n the min net load is '+str(df_stats['Load_net']['min']))
    print('the nuclear capacity is '+str(Capacity['pmax']['nuclear']))
    print('the max net load decrease '+str(df_stats_ramps['Load_net']['max']))
    print('the nuclear max ramp Down is '+str(Ramps_genType_df['rampDown']['nuclear']))
    print('are we in trouble for nuclear:'+ str(isNuclearInTrouble))
    print('WARNING: if the net demand is lowest than nuclear capacity, this will require to decrease nuclear production in the dispatch. If the demand decrease rate is stronger than the nuclear ramp, this might be tricky to do or infeasible')


    # In[185]:


    print('\n the max net load ramp up is '+str(df_stats_ramps['Load_net']['max']))
    print('the max generation ramp up is '+str(Ramps_genType_df['rampUp'].sum()))
    IsRampUpInTrouble=(df_stats_ramps['Load_net']['max'] > Ramps_genType_df['rampUp'].sum())


    # In[190]:
    
    RampDownNucHydro=0
    if('nuclear' in Ramps_genType_df.index):
        RampDownNucHydro+=Ramps_genType_df['rampDown']['nuclear']
    if('hydro' in Ramps_genType_df.index):
        RampDownNucHydro+=Ramps_genType_df['rampDown']['hydro']
    print('\n the max net load ramp down is '+str(df_stats_ramps['Load_net']['min']))
    print('the max nuclear + hydro ramp down is '+str(RampDownNucHydro))
    IsRampDownInTrouble=(df_stats_ramps['Load_net']['min'] > RampDownNucHydro)


    # In[179]:


    #import cufflinks as cf
    #import plotly.offline
    #plotly.offline.init_notebook_mode(connected=True) # for offline mode use
    #cf.go_offline()

    #Wind_df.iplot(kind='scatter', filename='cufflinks/cf-simple-line')
    #Load_net.iplot(kind='scatter', filename='cufflinks/cf-simple-line')

    return [isThermalInTrouble,isNuclearInTrouble,IsRampUpInTrouble,IsRampDownInTrouble]


def Aposteriori_renewableCapacityFactor_Checkers(env118_withoutchron,Capacity, chronics_path_gen):
    
    Wind_df=pd.DataFrame()
    Solar_df=pd.DataFrame()

    fileList=[f for f in os.listdir(chronics_path_gen) if not f.startswith('.')]
    for subpath in fileList:
        # Load consumption and prod
        this_path = os.path.join(chronics_path_gen, subpath)
        prod_p = pd.read_csv(os.path.join(this_path, 'prod_p.csv.bz2'), sep = ';')

       # Retrieve wind and solar from prod_p (Balthazar's generator)
        prod_p_wind = prod_p[[el for i, el in enumerate(env118_withoutchron.name_gen) if env118_withoutchron.gen_type[i] in ["wind"]]]
        total_p_wind=prod_p_wind.sum(axis=1)
        prod_p_solar = prod_p[[el for i, el in enumerate(env118_withoutchron.name_gen) if env118_withoutchron.gen_type[i] in ["solar"]]]
        total_p_solar=prod_p_solar.sum(axis=1) 

        # Demand for OPF (total - renewable)
        
        Wind_df[subpath]=total_p_wind
        Solar_df[subpath]=total_p_solar
        
    MaxSolar=Solar_df.max().max()
    MaxWind=Wind_df.max().max()
    
    solarCapacityFactor=Solar_df.mean().mean()/Capacity['pmax']['solar']#MaxSolar
    windCapacityFactor=Wind_df.mean().mean()/Capacity['pmax']['wind']#MaxWind
    
    print('\n the max wind production '+str(MaxWind))
    print('\n the expected max wind production was '+str(Capacity['pmax']['wind']))
    print('\n the max solar production '+str(MaxSolar))
    print('\n the expected max solar production was '+str(Capacity['pmax']['solar']))


    print('\n the solar capacity factor is: '+str(solarCapacityFactor))
    
    print('\n the expected solar capacity factor was: '+str(Capacity['capacity_factor']['solar']))
    
    print('\n the wind capacity factor is: '+str(windCapacityFactor))
    
    print('\n the expected wind capacity factor was: '+str(Capacity['capacity_factor']['wind']))
    
    return [solarCapacityFactor,windCapacityFactor]



from setuptools import setup


setup(name='Chronix2Grid',
      version='0.1.0rc3',
      description='A python package to generate "en-masse" chronics for loads and productions (thermal, renewable)',
      long_description='TODO',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
          "Intended Audience :: Developers",
          "Intended Audience :: Education",
          "Intended Audience :: Science/Research",
          "Natural Language :: English"
      ],
      keywords='ML powergrid optmization RL power-systems chronics generation production load network',
      author='Mario Jothy, Nicolas Megel, Vincent Renault',
      author_email=' mario.jothy@artelys.com',
      url="https://github.com/mjothy/chronix2grid",
      license='Mozilla Public License 2.0 (MPL 2.0)',
      packages=['chronix2grid', 'chronix2grid.generation', 'chronix2grid.kpi'],
      include_package_data=True,
      install_requires=["appdirs==1.4.3",
                        "attrs==19.3.0",
                        "backcall==0.1.0",
                        "bleach==3.1.1",
                        "branca==0.4.0",
                        "certifi==2019.11.28",
                        "chardet==3.0.4",
                        "click==7.1.1",
                        "colorama==0.4.3",
                        "colorlover==0.3.0",
                        "cufflinks==0.17.3",
                        "cycler==0.10.0",
                        "decorator==4.4.2",
                        "defusedxml==0.6.0",
                        "entrypoints==0.3",
                        "folium==0.10.1",
                        "Grid2Op==0.6.0",
                        "h5pyd==0.7.1",
                        "idna==2.9",
                        "importlib-metadata==1.5.0",
                        "ipykernel==5.1.4",
                        "ipython==7.13.0",
                        "ipython-genutils==0.2.0",
                        "ipywidgets==7.5.1",
                        "jedi==0.16.0",
                        "Jinja2==2.11.1",
                        "json5==0.9.3",
                        "jsonschema==3.2.0",
                        "jupyter==1.0.0",
                        "jupyter-client==6.0.0",
                        "jupyter-console==6.1.0",
                        "jupyter-core==4.6.3",
                        "jupyter-server==0.2.1",
                        "jupyterlab==2.0.1",
                        "jupyterlab-server==1.0.7",
                        "kiwisolver==1.1.0",
                        "llvmlite==0.31.0",
                        "MarkupSafe==1.1.1",
                        "matplotlib==3.1.3",
                        "mistune==0.8.4",
                        "nbconvert==5.6.1",
                        "nbformat==5.0.4",
                        "networkx==2.4",
                        "nose==1.3.7",
                        "notebook==6.0.3",
                        "numba==0.48.0",
                        "numexpr==2.7.1",
                        "numpy==1.18.2",
                        "packaging==20.3",
                        "pandapower==2.2.2",
                        "pandas==1.0.3",
                        "pandocfilters==1.4.2",
                        "parso==0.6.2",
                        "pathlib==1.0.1",
                        "pickleshare==0.7.5",
                        "plotly==4.5.2",
                        "ply==3.11",
                        "prometheus-client==0.7.1",
                        "prompt-toolkit==3.0.3",
                        "Pygments==2.5.2",
                        "Pyomo==5.6.8",
                        "pyparsing==2.4.6",
                        "pypsa==0.16.1",
                        "pyrsistent==0.15.7",
                        "python-dateutil==2.8.1",
                        "pytz==2019.3",
                        "PyUtilib==5.7.3",
                        "pywinpty==0.5.7",
                        "pyzmq==19.0.0",
                        "qtconsole==4.7.1",
                        "QtPy==1.9.0",
                        "requests==2.23.0",
                        "retrying==1.3.3",
                        "scipy==1.4.1",
                        "seaborn==0.10.0",
                        "Send2Trash==1.5.0",
                        "six==1.14.0",
                        "tables==3.6.1",
                        "terminado==0.8.3",
                        "testpath==0.4.4",
                        "tornado==6.0.3",
                        "tqdm==4.44.1",
                        "traitlets==4.3.3",
                        "urllib3==1.25.8",
                        "wcwidth==0.1.8",
                        "webencodings==0.5.1",
                        "widgetsnbextension==3.5.1",
                        "xlrd==1.2.0",
                        "zipp==3.1.0"
                        ],
      zip_safe=False,
      entry_points={'console_scripts': ['chronix2grid=chronix2grid.main:generate']}
)